import os
from itertools import product

import pytest

from together import (
    Together,
    BadRequestError,
    UnprocessableEntityError,
)
from together.types import Completion
from together.types.completion import (
    Choice,
    ChatCompletionUsage,
)

from ..constants import (
    completion_prompt_list,
    completion_test_model_list,
    moderation_test_model_list,
)
from .generate_hyperparameters import (
    random_min_p,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_top_k,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_top_p,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_max_tokens,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_temperature,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_presence_penalty,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_frequency_penalty,  # noqa: F401 # pyright: ignore[reportUnusedImport]
    random_repetition_penalty,  # noqa: F401 # pyright: ignore[reportUnusedImport]
)

STOP = ["</s>"]


class TestTogetherCompletion:
    @pytest.fixture
    def sync_together_client(self) -> Together:
        """
        Initialize object with mocked API key
        """
        TOGETHER_API_KEY = os.getenv("TOGETHER_API_KEY")
        return Together(api_key=TOGETHER_API_KEY)

    @pytest.mark.parametrize(
        "model,prompt",
        product(completion_test_model_list, completion_prompt_list),
    )
    def test_create(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
        random_max_tokens: int,
        random_temperature: float,
        random_top_p: float,
        random_top_k: int,
        random_presence_penalty: float,
        random_frequency_penalty: float,
        random_min_p: float,
    ) -> None:
        """
        Tests structure and typing
        """
        prompt = "The space robots have"

        # max_tokens should be a reasonable number for this test
        assert 0 < random_max_tokens < 1024

        assert 0 <= random_temperature <= 2

        assert 0 <= random_top_p <= 1

        assert 1 <= random_top_k <= 100

        assert -2 <= random_presence_penalty <= 2

        assert -2 <= random_frequency_penalty <= 2

        assert 0 <= random_min_p <= 1

        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=random_max_tokens,
            temperature=random_temperature,
            top_p=random_top_p,
            top_k=random_top_k,
            presence_penalty=random_presence_penalty,
            frequency_penalty=random_frequency_penalty,
            min_p=random_min_p,
            logit_bias={"1024": 10},
            echo=True,
        )

        assert isinstance(response, Completion)

        assert isinstance(response.id, str)
        assert isinstance(response.created, int)
        assert response.object == "text.completion"
        assert isinstance(response.choices, list)
        assert isinstance(response.choices[0], Choice)
        assert isinstance(response.choices[0].text, str)
        assert isinstance(response.prompt, list)
        assert isinstance(response.prompt[0].text, str)
        assert isinstance(response.usage, ChatCompletionUsage)
        assert isinstance(response.usage.prompt_tokens, int)
        assert isinstance(response.usage.completion_tokens, int)
        assert isinstance(response.usage.total_tokens, int)
        assert response.usage.prompt_tokens + response.usage.completion_tokens == response.usage.total_tokens

    @pytest.mark.parametrize(
        "model,prompt",
        product(completion_test_model_list, completion_prompt_list),
    )
    def test_prompt(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=10,
            echo=True,
        )

        assert isinstance(response, Completion)

        assert isinstance(response.prompt, list)
        assert response.prompt[0].text == prompt

    @pytest.mark.parametrize(
        "model,prompt",
        product(completion_test_model_list, completion_prompt_list),
    )
    def test_no_prompt(
        self,
        model: str,
        prompt: str,  # noqa
        sync_together_client: Together,
    ):
        with pytest.raises(TypeError):
            _ = sync_together_client.completions.create(  # pyright: ignore[reportCallIssue, reportUnknownVariableType]
                model=model,
                stop=STOP,
                max_tokens=10,
                echo=True,
            )

    @pytest.mark.parametrize(
        "model,prompt",
        product(completion_test_model_list, completion_prompt_list),
    )
    def test_model(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=10,
            echo=True,
        )

        assert isinstance(response, Completion)

    @pytest.mark.parametrize(
        "model,prompt",
        product(completion_test_model_list, completion_prompt_list),
    )
    def test_no_model(
        self,
        model: str,  # noqa
        prompt: str,
        sync_together_client: Together,
    ):
        with pytest.raises(TypeError):
            _ = sync_together_client.completions.create(  # pyright: ignore[reportCallIssue, reportUnknownVariableType]
                prompt=prompt,
                stop=STOP,
                max_tokens=10,
                echo=True,
            )

    @pytest.mark.parametrize(
        "model,prompt",
        product(completion_test_model_list, completion_prompt_list),
    )
    def test_max_tokens(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=1,
        )

        assert isinstance(response, Completion)

        assert response.usage is not None
        assert response.usage.completion_tokens == 1

    @pytest.mark.parametrize(
        "model,prompt,max_tokens",
        product(
            completion_test_model_list,
            completion_prompt_list,
            [200000, 400000, 500000],
        ),
    )
    def test_high_max_tokens(
        self,
        model: str,
        prompt: str,
        max_tokens: int,
        sync_together_client: Together,
    ):
        with pytest.raises(UnprocessableEntityError):
            _ = sync_together_client.completions.create(
                prompt=prompt,
                model=model,
                stop=STOP,
                max_tokens=max_tokens,
            )

    @pytest.mark.parametrize(
        "model,prompt",
        product(completion_test_model_list, completion_prompt_list),
    )
    def test_echo(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt, model=model, stop=STOP, max_tokens=1, echo=True, logprobs=1
        )

        assert isinstance(response, Completion)

        assert response.prompt is not None
        assert response.prompt[0].text == prompt
        assert response.prompt[0].logprobs is not None
        assert isinstance(response.prompt[0].logprobs.tokens, list)
        assert isinstance(response.prompt[0].logprobs.token_logprobs, list)

    @pytest.mark.parametrize(
        "model,prompt,n",
        product(completion_test_model_list, completion_prompt_list, [1, 2, 3, 4]),
    )
    def test_n(
        self,
        model: str,
        prompt: str,
        n: int,
        sync_together_client: Together,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt, model=model, stop=STOP, max_tokens=1, n=n, temperature=0.5
        )

        assert isinstance(response, Completion)

        assert len(response.choices) == n

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_high_n(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        MAX_N = 128
        n = MAX_N + 1

        with pytest.raises(BadRequestError):
            _ = sync_together_client.completions.create(
                prompt=prompt,
                model=model,
                stop=STOP,
                max_tokens=1,
                n=n,
                temperature=0.1,
            )

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_n_with_no_sample(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        MAX_N = 128
        n = MAX_N + 1

        with pytest.raises(BadRequestError):
            _ = sync_together_client.completions.create(
                prompt=prompt,
                model=model,
                stop=STOP,
                max_tokens=1,
                n=n,
            )

    @pytest.mark.parametrize(
        "model,prompt,safety_model",
        product(
            completion_test_model_list,
            completion_prompt_list,
            moderation_test_model_list,
        ),
    )
    def test_safety_model(
        self,
        model: str,
        prompt: str,
        safety_model: str,
        sync_together_client: Together,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=1,
            safety_model=safety_model,
        )

        assert isinstance(response, Completion)

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_repetition_penalty(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
        random_repetition_penalty: float,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=10,
            repetition_penalty=random_repetition_penalty,
        )

        assert isinstance(response, Completion)

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_presence_penalty(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
        random_presence_penalty: float,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=10,
            presence_penalty=random_presence_penalty,
        )

        assert isinstance(response, Completion)

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_high_presence_penalty(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        with pytest.raises(BadRequestError):
            _ = sync_together_client.completions.create(
                prompt=prompt,
                model=model,
                stop=STOP,
                max_tokens=10,
                presence_penalty=2.1,
            )

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_frequency_penalty(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
        random_frequency_penalty: float,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=10,
            frequency_penalty=random_frequency_penalty,
        )

        assert isinstance(response, Completion)

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_high_frequency_penalty(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        with pytest.raises(BadRequestError):
            _ = sync_together_client.completions.create(
                prompt=prompt,
                model=model,
                stop=STOP,
                max_tokens=10,
                frequency_penalty=2.1,
            )

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_min_p(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
        random_min_p: float,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=10,
            min_p=random_min_p,
        )

        assert isinstance(response, Completion)

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_high_min_p(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        with pytest.raises(BadRequestError):
            _ = sync_together_client.completions.create(
                prompt=prompt,
                model=model,
                stop=STOP,
                max_tokens=10,
                min_p=1.1,
            )

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_logit_bias(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=1,
            logit_bias={"1024": 10},
        )

        assert isinstance(response, Completion)

    @pytest.mark.parametrize(
        "model,prompt",
        product(
            completion_test_model_list,
            completion_prompt_list,
        ),
    )
    def test_seed(
        self,
        model: str,
        prompt: str,
        sync_together_client: Together,
    ):
        response = sync_together_client.completions.create(
            prompt=prompt,
            model=model,
            stop=STOP,
            max_tokens=1,
            seed=4242,
        )

        assert isinstance(response, Completion)
        assert response.choices[0].seed == 4242



from typing import Any, Dict, List
from textwrap import wrap

import click
from tabulate import tabulate

from together import Together
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.pass_context
@click.argument("fine_tune_id", type=str, required=True)
@handle_api_errors("Fine-tuning")
def list_events(ctx: click.Context, fine_tune_id: str) -> None:
    """List fine-tuning events"""
    client: Together = ctx.obj

    response = client.fine_tuning.list_events(fine_tune_id)

    response.data = response.data or []

    display_list: List[Dict[str, Any]] = []
    for i in response.data:
        display_list.append(
            {
                "Message": "\n".join(wrap(i.message or "", width=50)),
                "Type": i.type,
                "Created At": i.created_at,
                "Hash": i.hash,
            }
        )
    table = tabulate(display_list, headers="keys", tablefmt="grid", showindex=True)

    click.echo(table)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from together import Together, AsyncTogether
from tests.utils import assert_matches_type
from together.types import ImageFile

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestImages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_generate(self, client: Together) -> None:
        image = client.images.generate(
            model="black-forest-labs/FLUX.1-schnell",
            prompt="cat floating in space, cinematic",
        )
        assert_matches_type(ImageFile, image, path=["response"])

    @parametrize
    def test_method_generate_with_all_params(self, client: Together) -> None:
        image = client.images.generate(
            model="black-forest-labs/FLUX.1-schnell",
            prompt="cat floating in space, cinematic",
            disable_safety_checker=True,
            guidance_scale=0,
            height=0,
            image_loras=[
                {
                    "path": "path",
                    "scale": 0,
                }
            ],
            image_url="image_url",
            n=0,
            negative_prompt="negative_prompt",
            output_format="jpeg",
            reference_images=["string"],
            response_format="base64",
            seed=0,
            steps=0,
            width=0,
        )
        assert_matches_type(ImageFile, image, path=["response"])

    @parametrize
    def test_raw_response_generate(self, client: Together) -> None:
        response = client.images.with_raw_response.generate(
            model="black-forest-labs/FLUX.1-schnell",
            prompt="cat floating in space, cinematic",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image = response.parse()
        assert_matches_type(ImageFile, image, path=["response"])

    @parametrize
    def test_streaming_response_generate(self, client: Together) -> None:
        with client.images.with_streaming_response.generate(
            model="black-forest-labs/FLUX.1-schnell",
            prompt="cat floating in space, cinematic",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image = response.parse()
            assert_matches_type(ImageFile, image, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncImages:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_generate(self, async_client: AsyncTogether) -> None:
        image = await async_client.images.generate(
            model="black-forest-labs/FLUX.1-schnell",
            prompt="cat floating in space, cinematic",
        )
        assert_matches_type(ImageFile, image, path=["response"])

    @parametrize
    async def test_method_generate_with_all_params(self, async_client: AsyncTogether) -> None:
        image = await async_client.images.generate(
            model="black-forest-labs/FLUX.1-schnell",
            prompt="cat floating in space, cinematic",
            disable_safety_checker=True,
            guidance_scale=0,
            height=0,
            image_loras=[
                {
                    "path": "path",
                    "scale": 0,
                }
            ],
            image_url="image_url",
            n=0,
            negative_prompt="negative_prompt",
            output_format="jpeg",
            reference_images=["string"],
            response_format="base64",
            seed=0,
            steps=0,
            width=0,
        )
        assert_matches_type(ImageFile, image, path=["response"])

    @parametrize
    async def test_raw_response_generate(self, async_client: AsyncTogether) -> None:
        response = await async_client.images.with_raw_response.generate(
            model="black-forest-labs/FLUX.1-schnell",
            prompt="cat floating in space, cinematic",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        image = await response.parse()
        assert_matches_type(ImageFile, image, path=["response"])

    @parametrize
    async def test_streaming_response_generate(self, async_client: AsyncTogether) -> None:
        async with async_client.images.with_streaming_response.generate(
            model="black-forest-labs/FLUX.1-schnell",
            prompt="cat floating in space, cinematic",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            image = await response.parse()
            assert_matches_type(ImageFile, image, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .tool_choice import ToolChoice
from .chat.chat_completion_usage import ChatCompletionUsage

__all__ = ["CompletionChunk", "Token", "Choice", "ChoiceDelta", "ChoiceDeltaFunctionCall"]


class Token(BaseModel):
    id: int

    logprob: float

    special: bool

    text: str


class ChoiceDeltaFunctionCall(BaseModel):
    arguments: str

    name: str


class ChoiceDelta(BaseModel):
    role: Literal["system", "user", "assistant", "function", "tool"]

    content: Optional[str] = None

    function_call: Optional[ChoiceDeltaFunctionCall] = None

    reasoning: Optional[str] = None

    token_id: Optional[int] = None

    tool_calls: Optional[List[ToolChoice]] = None


class Choice(BaseModel):
    index: int

    delta: Optional[ChoiceDelta] = None

    text: Optional[str] = None


class CompletionChunk(BaseModel):
    id: str

    token: Token

    choices: List[Choice]

    finish_reason: Optional[Literal["stop", "eos", "length", "tool_calls", "function_call"]] = None

    usage: Optional[ChatCompletionUsage] = None

    created: Optional[int] = None

    object: Optional[Literal["completion.chunk"]] = None

    seed: Optional[int] = None



from __future__ import annotations

import sys
from typing import Any, Dict

import click

from together import Together
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.argument("endpoint-id", required=True)
@click.option(
    "--display-name",
    help="A new human-readable name for the endpoint",
)
@click.option(
    "--min-replicas",
    type=int,
    help="New minimum number of replicas to maintain",
)
@click.option(
    "--max-replicas",
    type=int,
    help="New maximum number of replicas to scale up to",
)
@click.option(
    "--inactive-timeout",
    type=int,
    help="Number of minutes of inactivity after which the endpoint will be automatically stopped. Set to 0 to disable.",
)
@click.pass_obj
@handle_api_errors("Endpoints")
def update(
    client: Together,
    endpoint_id: str,
    display_name: str | None,
    min_replicas: int | None,
    max_replicas: int | None,
    inactive_timeout: int | None,
) -> None:
    """Update a dedicated inference endpoint's configuration."""
    if not any([display_name, min_replicas, max_replicas, inactive_timeout]):
        click.echo("Error: At least one update option must be specified", err=True)
        sys.exit(1)

    # Build kwargs for the update
    kwargs: Dict[str, Any] = {}
    if display_name is not None:
        kwargs["display_name"] = display_name

    if min_replicas is not None or max_replicas is not None:
        kwargs["autoscaling"] = {}
        if min_replicas is not None:
            kwargs["autoscaling"]["min_replicas"] = min_replicas
        if max_replicas is not None:
            kwargs["autoscaling"]["max_replicas"] = max_replicas

    if inactive_timeout is not None:
        kwargs["inactive_timeout"] = inactive_timeout

    client.endpoints.update(endpoint_id, **kwargs)

    # Print what was updated
    click.echo("Updated endpoint configuration:", err=True)
    if display_name:
        click.echo(f"  Display name: {display_name}", err=True)
    if min_replicas:
        click.echo(f"  Min replicas: {min_replicas}", err=True)
    if max_replicas:
        click.echo(f"  Max replicas: {max_replicas}", err=True)
    if inactive_timeout is not None:
        click.echo(f"  Inactive timeout: {inactive_timeout} minutes", err=True)

    click.echo("Successfully updated endpoint", err=True)
    click.echo(endpoint_id)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["FinetuneEventType"]

FinetuneEventType: TypeAlias = Literal[
    "job_pending",
    "job_start",
    "job_stopped",
    "model_downloading",
    "model_download_complete",
    "training_data_downloading",
    "training_data_download_complete",
    "validation_data_downloading",
    "validation_data_download_complete",
    "wandb_init",
    "training_start",
    "checkpoint_save",
    "billing_limit",
    "epoch_complete",
    "training_complete",
    "model_compressing",
    "model_compression_complete",
    "model_uploading",
    "model_upload_complete",
    "job_complete",
    "job_error",
    "cancel_requested",
    "job_restarted",
    "refund",
    "warning",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .cluster_storage import ClusterStorage as ClusterStorage
from .storage_create_params import StorageCreateParams as StorageCreateParams
from .storage_list_response import StorageListResponse as StorageListResponse
from .storage_update_params import StorageUpdateParams as StorageUpdateParams
from .storage_delete_response import StorageDeleteResponse as StorageDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from together import Together, AsyncTogether
from tests.utils import assert_matches_type
from together.types import VideoJob

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVideos:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Together) -> None:
        video = client.videos.create(
            model="model",
        )
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Together) -> None:
        video = client.videos.create(
            model="model",
            fps=0,
            frame_images=[
                {
                    "input_image": "input_image",
                    "frame": 0,
                }
            ],
            guidance_scale=0,
            height=0,
            negative_prompt="negative_prompt",
            output_format="MP4",
            output_quality=0,
            prompt="x",
            reference_images=["string"],
            seconds="seconds",
            seed=0,
            steps=10,
            width=0,
        )
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Together) -> None:
        response = client.videos.with_raw_response.create(
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        video = response.parse()
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Together) -> None:
        with client.videos.with_streaming_response.create(
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            video = response.parse()
            assert_matches_type(VideoJob, video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Together) -> None:
        video = client.videos.retrieve(
            "id",
        )
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Together) -> None:
        response = client.videos.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        video = response.parse()
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Together) -> None:
        with client.videos.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            video = response.parse()
            assert_matches_type(VideoJob, video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Together) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.videos.with_raw_response.retrieve(
                "",
            )


class TestAsyncVideos:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncTogether) -> None:
        video = await async_client.videos.create(
            model="model",
        )
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTogether) -> None:
        video = await async_client.videos.create(
            model="model",
            fps=0,
            frame_images=[
                {
                    "input_image": "input_image",
                    "frame": 0,
                }
            ],
            guidance_scale=0,
            height=0,
            negative_prompt="negative_prompt",
            output_format="MP4",
            output_quality=0,
            prompt="x",
            reference_images=["string"],
            seconds="seconds",
            seed=0,
            steps=10,
            width=0,
        )
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTogether) -> None:
        response = await async_client.videos.with_raw_response.create(
            model="model",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        video = await response.parse()
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTogether) -> None:
        async with async_client.videos.with_streaming_response.create(
            model="model",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            video = await response.parse()
            assert_matches_type(VideoJob, video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTogether) -> None:
        video = await async_client.videos.retrieve(
            "id",
        )
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTogether) -> None:
        response = await async_client.videos.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        video = await response.parse()
        assert_matches_type(VideoJob, video, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTogether) -> None:
        async with async_client.videos.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            video = await response.parse()
            assert_matches_type(VideoJob, video, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTogether) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.videos.with_raw_response.retrieve(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["EvalCreateResponse"]


class EvalCreateResponse(BaseModel):
    status: Optional[Literal["pending"]] = None
    """Initial status of the job"""

    workflow_id: Optional[str] = None
    """The ID of the created evaluation job"""



import json

import click

from together import Together
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.pass_context
@click.argument("id", type=str, required=True)
@handle_api_errors("Files")
def retrieve(ctx: click.Context, id: str) -> None:
    """Upload file"""

    client: Together = ctx.obj

    response = client.files.retrieve(id=id)

    click.echo(json.dumps(response.model_dump(exclude_none=True), indent=4))



from typing import List

completion_test_model_list: List[str] = [
    "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
]
chat_test_model_list: List[str] = []
embedding_test_model_list: List[str] = []
image_test_model_list: List[str] = []
moderation_test_model_list: List[str] = []

LLAMA_PROMPT = """Llamas that are well-socialized and trained to halter and lead after weaning are very friendly and pleasant to be around. They are extremely curious and most will approach people easily. However, llamas that are bottle-fed or over-socialized and over-handled as youth will become extremely difficult to handle when mature, when they will begin to treat humans as they treat each other, which is characterized by bouts of spitting, kicking and neck wrestling.
Llamas are now utilized as certified therapy animals in nursing homes and hospitals. Rojo the Llama, located in the Pacific Northwest was certified in 2008. The Mayo Clinic says animal-assisted therapy can reduce pain, depression, anxiety, and fatigue. This type of therapy is growing in popularity, and there are several organizations throughout the United States that participate.
When correctly reared, llamas spitting at a human is a rare thing. Llamas are very social herd animals, however, and do sometimes spit at each other as a way of disciplining lower-ranked llamas in the herd. A llama's social rank in a herd is never static. They can always move up or down in the social ladder by picking small fights. This is usually done between males to see which will become dominant. Their fights are visually dramatic, with spitting, ramming each other with their chests, neck wrestling and kicking, mainly to knock the other off balance. The females are usually only seen spitting as a means of controlling other herd members. One may determine how agitated the llama is by the materials in the spit. The more irritated the llama is, the further back into each of the three stomach compartments it will try to draw materials from for its spit.
While the social structure might always be changing, they live as a family and they do take care of each other. If one notices a strange noise or feels threatened, an alarm call - a loud, shrill sound which rhythmically rises and falls - is sent out and all others become alert. They will often hum to each other as a form of communication.
The sound of the llama making groaning noises or going "mwa" is often a sign of fear or anger. Unhappy or agitated llamas will lay their ears back, while ears being perked upwards is a sign of happiness or curiosity.
An "orgle" is the mating sound of a llama or alpaca, made by the sexually aroused male. The sound is reminiscent of gargling, but with a more forceful, buzzing edge. Males begin the sound when they become aroused and continue throughout copulation.
Using llamas as livestock guards in North America began in the early 1980s, and some sheep producers have used llamas successfully since then. Some would even use them to guard their smaller cousins, the alpaca. They are used most commonly in the western regions of the United States, where larger predators, such as coyotes and feral dogs, are prevalent. Typically, a single gelding (castrated male) is used."""

completion_prompt_list: List[str] = [
    "The quick brown fox jumps over the lazy dog. The quick brown fox jumps over the lazy dog.",
    "hi," * 25,
    LLAMA_PROMPT,
]



import click

from .list import list
from .create import create
from .delete import delete
from .retrieve import retrieve


@click.group()
@click.pass_context
def storage(ctx: click.Context) -> None:
    """Clusters Storage API commands"""
    pass


storage.add_command(create)
storage.add_command(retrieve)
storage.add_command(delete)
storage.add_command(list)



import json as json_lib
from typing import Any, Dict, List, Optional

import click
from tabulate import tabulate

from together import Together, omit
from together._response import APIResponse as APIResponse
from together.lib.cli.api._utils import handle_api_errors
from together.lib.utils.serializer import datetime_serializer


@click.command()
@click.option(
    "--type",
    type=click.Choice(["dedicated"]),
    help="Filter models by type (dedicated: models that can be deployed as dedicated endpoints)",
)
@click.option(
    "--json",
    is_flag=True,
    help="Output in JSON format",
)
@click.pass_context
@handle_api_errors("Models")
def list(ctx: click.Context, type: Optional[str], json: bool) -> None:
    """List models"""
    client: Together = ctx.obj

    models_list = client.models.list(dedicated=type == "dedicated" if type else omit)

    if json:
        items = [model.model_dump() for model in models_list]
        click.echo(json_lib.dumps(items, indent=2, default=datetime_serializer))
        return

    display_list: List[Dict[str, Any]] = []
    for model in sorted(models_list, key=lambda x: x.type):
        price_parts: List[str] = []

        # Only show pricing if a value actually exists
        if model.pricing and model.pricing.input > 0 and model.pricing.output > 0:
            price_parts.append(f"${model.pricing.input:.2f}")
            price_parts.append(f"${model.pricing.output:.2f}")

        display_list.append(
            {
                "Model": model.id,
                "Type": model.type,
                "Context length": model.context_length if model.context_length else None,
                "Price per 1M Tokens (input/output)": "/".join(price_parts),
            }
        )

    click.echo(tabulate(display_list, headers="keys"))



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["CompletionCreateParamsBase", "CompletionCreateParamsNonStreaming", "CompletionCreateParamsStreaming"]


class CompletionCreateParamsBase(TypedDict, total=False):
    model: Required[
        Union[
            Literal[
                "meta-llama/Llama-2-70b-hf",
                "mistralai/Mistral-7B-v0.1",
                "mistralai/Mixtral-8x7B-v0.1",
                "Meta-Llama/Llama-Guard-7b",
            ],
            str,
        ]
    ]
    """The name of the model to query.

    [See all of Together AI's chat models](https://docs.together.ai/docs/serverless-models#chat-models)
    """

    prompt: Required[str]
    """A string providing context for the model to complete."""

    echo: bool
    """If true, the response will contain the prompt.

    Can be used with `logprobs` to return prompt logprobs.
    """

    frequency_penalty: float
    """
    A number between -2.0 and 2.0 where a positive value decreases the likelihood of
    repeating tokens that have already been mentioned.
    """

    logit_bias: Dict[str, float]
    """Adjusts the likelihood of specific tokens appearing in the generated output."""

    logprobs: int
    """
    An integer between 0 and 20 of the top k tokens to return log probabilities for
    at each generation step, instead of just the sampled token. Log probabilities
    help assess model confidence in token predictions.
    """

    max_tokens: int
    """The maximum number of tokens to generate."""

    min_p: float
    """A number between 0 and 1 that can be used as an alternative to top-p and top-k."""

    n: int
    """The number of completions to generate for each prompt."""

    presence_penalty: float
    """
    A number between -2.0 and 2.0 where a positive value increases the likelihood of
    a model talking about new topics.
    """

    repetition_penalty: float
    """
    A number that controls the diversity of generated text by reducing the
    likelihood of repeated sequences. Higher values decrease repetition.
    """

    safety_model: Union[Literal["Meta-Llama/Llama-Guard-7b"], str]
    """The name of the moderation model used to validate tokens.

    Choose from the available moderation models found
    [here](https://docs.together.ai/docs/inference-models#moderation-models).
    """

    seed: int
    """Seed value for reproducibility."""

    stop: SequenceNotStr[str]
    """A list of string sequences that will truncate (stop) inference text output.

    For example, "</s>" will stop generation as soon as the model generates the
    given token.
    """

    temperature: float
    """
    A decimal number from 0-1 that determines the degree of randomness in the
    response. A temperature less than 1 favors more correctness and is appropriate
    for question answering or summarization. A value closer to 1 introduces more
    randomness in the output.
    """

    top_k: int
    """
    An integer that's used to limit the number of choices for the next predicted
    word or token. It specifies the maximum number of tokens to consider at each
    step, based on their probability of occurrence. This technique helps to speed up
    the generation process and can improve the quality of the generated text by
    focusing on the most likely options.
    """

    top_p: float
    """
    A percentage (also called the nucleus parameter) that's used to dynamically
    adjust the number of choices for each predicted token based on the cumulative
    probabilities. It specifies a probability threshold below which all less likely
    tokens are filtered out. This technique helps maintain diversity and generate
    more fluent and natural-sounding text.
    """


class CompletionCreateParamsNonStreaming(CompletionCreateParamsBase, total=False):
    stream: Literal[False]
    """
    If true, stream tokens as Server-Sent Events as the model generates them instead
    of waiting for the full model response. The stream terminates with
    `data: [DONE]`. If false, return a single JSON object containing the results.
    """


class CompletionCreateParamsStreaming(CompletionCreateParamsBase):
    stream: Required[Literal[True]]
    """
    If true, stream tokens as Server-Sent Events as the model generates them instead
    of waiting for the full model response. The stream terminates with
    `data: [DONE]`. If false, return a single JSON object containing the results.
    """


CompletionCreateParams = Union[CompletionCreateParamsNonStreaming, CompletionCreateParamsStreaming]



from __future__ import annotations

import json as json_lib

import click

from together import Together, omit
from together.lib.cli.api._utils import handle_api_errors
from together.lib.utils.serializer import datetime_serializer


@click.command()
@click.option("--model", help="Filter hardware options by model")
@click.option("--json", is_flag=True, help="Print output in JSON format")
@click.option(
    "--available",
    is_flag=True,
    help="Print only available hardware options (can only be used if model is passed in)",
)
@click.pass_obj
@handle_api_errors("Endpoints")
def hardware(client: Together, model: str | None, json: bool, available: bool) -> None:
    """List all available hardware options, optionally filtered by model."""
    message = "Available hardware options:" if available else "All hardware options:"
    click.echo(message, err=True)
    hardware_options = client.hardware.list(model=model or omit)
    # hardware_options = client.endpoints.list_hardware(model)
    if available:
        hardware_options.data = [
            hardware
            for hardware in hardware_options.data
            if hardware.availability is not None and hardware.availability.status == "available"
        ]

    if json:
        json_output = [hardware.model_dump() for hardware in hardware_options.data]
        click.echo(json_lib.dumps(json_output, default=datetime_serializer, indent=2))
    else:
        for hardware in hardware_options.data:
            click.echo(f"  {hardware.id}", err=True)



import json
from typing import Any, Dict, Union, Literal, Optional, cast

import click

from together import Together, TogetherError
from together.lib.cli.api._utils import handle_api_errors
from together.types.eval_create_params import (
    ParametersEvaluationScoreParameters,
    ParametersEvaluationCompareParameters,
    ParametersEvaluationClassifyParameters,
    ParametersEvaluationScoreParametersJudge,
    ParametersEvaluationCompareParametersJudge,
    ParametersEvaluationClassifyParametersJudge,
    ParametersEvaluationScoreParametersModelToEvaluate,
    ParametersEvaluationClassifyParametersModelToEvaluate,
    ParametersEvaluationCompareParametersModelAEvaluationModelRequest,
    ParametersEvaluationCompareParametersModelBEvaluationModelRequest,
)


@click.command()
@click.option(
    "--type",
    type=click.Choice(["classify", "score", "compare"]),
    required=True,
    help="Type of evaluation to create.",
)
@click.option(
    "--judge-model",
    type=str,
    required=True,
    help="Name or URL of the judge model to use for evaluation.",
)
@click.option(
    "--judge-model-source",
    type=click.Choice(["serverless", "dedicated", "external"]),
    required=True,
    help="Source of the judge model.",
)
@click.option(
    "--judge-external-api-token",
    type=str,
    required=False,
    help="Optional external API token for the judge model.",
)
@click.option(
    "--judge-external-base-url",
    type=str,
    required=False,
    help="Optional external base URLs for the judge model.",
)
@click.option(
    "--judge-system-template",
    type=str,
    required=True,
    help="System template for the judge model.",
)
@click.option(
    "--input-data-file-path",
    type=str,
    required=True,
    help="Path to the input data file.",
)
@click.option(
    "--model-field",
    type=str,
    help="Name of the field in the input file contaning text generated by the model."
    "Can not be used when model-a-name and other model config parameters are specified",
)
@click.option(
    "--model-to-evaluate",
    type=str,
    help="Model name when using the detailed config",
)
@click.option(
    "--model-to-evaluate-source",
    type=click.Choice(["serverless", "dedicated", "external"]),
    help="Source of the model to evaluate.",
)
@click.option(
    "--model-to-evaluate-external-api-token",
    type=str,
    help="Optional external API token for the model to evaluate.",
)
@click.option(
    "--model-to-evaluate-external-base-url",
    type=str,
    help="Optional external base URL for the model to evaluate.",
)
@click.option(
    "--model-to-evaluate-max-tokens",
    type=int,
    help="Max tokens for model-to-evaluate",
)
@click.option(
    "--model-to-evaluate-temperature",
    type=float,
    help="Temperature for model-to-evaluate",
)
@click.option(
    "--model-to-evaluate-system-template",
    type=str,
    help="System template for model-to-evaluate",
)
@click.option(
    "--model-to-evaluate-input-template",
    type=str,
    help="Input template for model-to-evaluate",
)
@click.option(
    "--labels",
    type=str,
    help="Classification labels - comma-separated list",
)
@click.option(
    "--pass-labels",
    type=str,
    help="Labels considered as passing (required for classify type). A comma-separated list.",
)
@click.option(
    "--min-score",
    type=float,
    help="Minimum score value (required for score type).",
)
@click.option(
    "--max-score",
    type=float,
    help="Maximum score value (required for score type).",
)
@click.option(
    "--pass-threshold",
    type=float,
    help="Threshold score for passing (required for score type).",
)
@click.option(
    "--model-a-field",
    type=str,
    help="Name of the field in the input file containing text generated by Model A. \
        Can not be used when model-a-name and other model config parameters are specified",
)
@click.option(
    "--model-a",
    type=str,
    help="Model name or URL for model A when using detailed config.",
)
@click.option(
    "--model-a-source",
    type=click.Choice(["serverless", "dedicated", "external"]),
    help="Source of model A.",
)
@click.option(
    "--model-a-external-api-token",
    type=str,
    help="Optional external API token for model A.",
)
@click.option(
    "--model-a-external-base-url",
    type=str,
    help="Optional external base URL for model A.",
)
@click.option(
    "--model-a-max-tokens",
    type=int,
    help="Max tokens for model A.",
)
@click.option(
    "--model-a-temperature",
    type=float,
    help="Temperature for model A.",
)
@click.option(
    "--model-a-system-template",
    type=str,
    help="System template for model A.",
)
@click.option(
    "--model-a-input-template",
    type=str,
    help="Input template for model A.",
)
@click.option(
    "--model-b-field",
    type=str,
    help="Name of the field in the input file containing text generated by Model B.\
          Can not be used when model-b-name and other model config parameters are specified",
)
@click.option(
    "--model-b",
    type=str,
    help="Model name or URL for model B when using detailed config.",
)
@click.option(
    "--model-b-source",
    type=click.Choice(["serverless", "dedicated", "external"]),
    help="Source of model B.",
)
@click.option(
    "--model-b-external-api-token",
    type=str,
    help="Optional external API token for model B.",
)
@click.option(
    "--model-b-external-base-url",
    type=str,
    help="Optional external base URL for model B.",
)
@click.option(
    "--model-b-max-tokens",
    type=int,
    help="Max tokens for model B.",
)
@click.option(
    "--model-b-temperature",
    type=float,
    help="Temperature for model B.",
)
@click.option(
    "--model-b-system-template",
    type=str,
    help="System template for model B.",
)
@click.option(
    "--model-b-input-template",
    type=str,
    help="Input template for model B.",
)
@click.pass_context
@handle_api_errors("Evals")
def create(
    ctx: click.Context,
    type: Literal["classify", "score", "compare"],
    judge_model: str,
    judge_model_source: Literal["serverless", "dedicated", "external"],
    judge_system_template: str,
    judge_external_api_token: Optional[str],
    judge_external_base_url: Optional[str],
    input_data_file_path: str,
    model_field: Optional[str],
    model_to_evaluate: Optional[str],
    model_to_evaluate_source: Optional[str],
    model_to_evaluate_external_api_token: Optional[str],
    model_to_evaluate_external_base_url: Optional[str],
    model_to_evaluate_max_tokens: Optional[int],
    model_to_evaluate_temperature: Optional[float],
    model_to_evaluate_system_template: Optional[str],
    model_to_evaluate_input_template: Optional[str],
    labels: str,
    pass_labels: str,
    min_score: Optional[float],
    max_score: Optional[float],
    pass_threshold: Optional[float],
    model_a_field: Optional[str],
    model_a: Optional[str],
    model_a_source: Optional[str],
    model_a_external_api_token: Optional[str],
    model_a_external_base_url: Optional[str],
    model_a_max_tokens: Optional[int],
    model_a_temperature: Optional[float],
    model_a_system_template: Optional[str],
    model_a_input_template: Optional[str],
    model_b_field: Optional[str],
    model_b: Optional[str],
    model_b_source: Optional[str],
    model_b_external_api_token: Optional[str],
    model_b_external_base_url: Optional[str],
    model_b_max_tokens: Optional[int],
    model_b_temperature: Optional[float],
    model_b_system_template: Optional[str],
    model_b_input_template: Optional[str],
) -> None:
    """Create a new evaluation job"""

    client: Together = ctx.obj

    # Convert strings to lists for labels
    labels_list = labels.split(",") if labels else None
    pass_labels_list = pass_labels.split(",") if pass_labels else None

    # Build model configurations
    model_to_evaluate_final: Union[Dict[str, Any], None, str] = None

    # Check if any config parameters are provided
    config_params_provided = any(
        [
            model_to_evaluate,
            model_to_evaluate_source,
            model_to_evaluate_max_tokens,
            model_to_evaluate_temperature,
            model_to_evaluate_system_template,
            model_to_evaluate_input_template,
        ]
    )

    if model_field:
        # Simple mode: model_field is provided
        if config_params_provided:
            raise click.BadParameter(
                "Cannot specify both --model-field and --model-to-evaluate-* parameters. "
                "Use either --model-field alone if your input file has pre-generated responses, "
                "or config parameters if you want to generate it on our end"
            )
        model_to_evaluate_final = model_field
    elif config_params_provided:
        # Config mode: config parameters are provided
        model_to_evaluate_final = {
            "model": model_to_evaluate,
            "model_source": model_to_evaluate_source,
            "max_tokens": model_to_evaluate_max_tokens,
            "temperature": model_to_evaluate_temperature,
            "system_template": model_to_evaluate_system_template,
            "input_template": model_to_evaluate_input_template,
        }
        if model_to_evaluate_external_api_token:
            model_to_evaluate_final["external_api_token"] = model_to_evaluate_external_api_token
        if model_to_evaluate_external_base_url:
            model_to_evaluate_final["external_base_url"] = model_to_evaluate_external_base_url

    # Build model-a configuration
    model_a_final: Union[Dict[str, Any], None, str] = None
    model_a_config_params = [
        model_a,
        model_a_source,
        model_a_max_tokens,
        model_a_temperature,
        model_a_system_template,
        model_a_input_template,
    ]

    if model_a_field is not None:
        # Simple mode: model_a_field is provided
        if any(model_a_config_params):
            raise click.BadParameter(
                "Cannot specify both --model-a-field and config parameters (--model-a-name, etc.). "
                "Use either --model-a-field alone if your input file has pre-generated responses, "
                "or config parameters if you want to generate it on our end"
            )
        model_a_final = model_a_field
    elif any(model_a_config_params):
        # Config mode: config parameters are provided
        model_a_final = {
            "model": model_a,
            "model_source": model_a_source,
            "max_tokens": model_a_max_tokens,
            "temperature": model_a_temperature,
            "system_template": model_a_system_template,
            "input_template": model_a_input_template,
        }
        if model_a_external_api_token:
            model_a_final["external_api_token"] = model_a_external_api_token
        if model_a_external_base_url:
            model_a_final["external_base_url"] = model_a_external_base_url

    # Build model-b configuration
    model_b_final: Union[Dict[str, Any], None, str] = None
    model_b_config_params = [
        model_b,
        model_b_source,
        model_b_max_tokens,
        model_b_temperature,
        model_b_system_template,
        model_b_input_template,
    ]

    if model_b_field is not None:
        # Simple mode: model_b_field is provided
        if any(model_b_config_params):
            raise click.BadParameter(
                "Cannot specify both --model-b-field and config parameters (--model-b-name, etc.). "
                "Use either --model-b-field alone if your input file has pre-generated responses, "
                "or config parameters if you want to generate it on our end"
            )
        model_b_final = model_b_field
    elif any(model_b_config_params):
        # Config mode: config parameters are provided
        model_b_final = {
            "model": model_b,
            "model_source": model_b_source,
            "max_tokens": model_b_max_tokens,
            "temperature": model_b_temperature,
            "system_template": model_b_system_template,
            "input_template": model_b_input_template,
        }
        if model_b_external_api_token:
            model_b_final["external_api_token"] = model_b_external_api_token
        if model_b_external_base_url:
            model_b_final["external_base_url"] = model_b_external_base_url

    judge_config = _build_judge(
        type, judge_model, judge_model_source, judge_system_template, judge_external_api_token, judge_external_base_url
    )

    if type == "classify":
        response = client.evals.create(
            type=type,
            parameters=ParametersEvaluationClassifyParameters(
                input_data_file_path=input_data_file_path,
                judge=judge_config,
                labels=labels_list or [],
                pass_labels=pass_labels_list or [],
                model_to_evaluate=cast(ParametersEvaluationClassifyParametersModelToEvaluate, model_to_evaluate_final),
            ),
        )
    elif type == "score":
        if max_score is None or min_score is None or pass_threshold is None:
            raise TogetherError("max_score, min_score, and pass_threshold are required for score type")

        response = client.evals.create(
            type="score",
            parameters=ParametersEvaluationScoreParameters(
                input_data_file_path=input_data_file_path,
                judge=judge_config,
                max_score=max_score,
                min_score=min_score,
                pass_threshold=pass_threshold,
                model_to_evaluate=cast(ParametersEvaluationScoreParametersModelToEvaluate, model_to_evaluate_final),
            ),
        )
    elif type == "compare":
        response = client.evals.create(
            type=type,
            parameters=ParametersEvaluationCompareParameters(
                input_data_file_path=input_data_file_path,
                judge=judge_config,
                model_a=cast(ParametersEvaluationCompareParametersModelAEvaluationModelRequest, model_a_final),
                model_b=cast(ParametersEvaluationCompareParametersModelBEvaluationModelRequest, model_b_final),
            ),
        )

    click.echo(json.dumps(response.model_dump(exclude_none=True), indent=4))


def _build_judge(
    type: Literal["classify", "score", "compare"],
    judge_model: str,
    judge_model_source: Literal["serverless", "dedicated", "external"],
    judge_system_template: str,
    judge_external_api_token: Optional[str],
    judge_external_base_url: Optional[str],
) -> ParametersEvaluationClassifyParametersJudge:
    if type == "classify":
        judge_config = ParametersEvaluationClassifyParametersJudge(
            model=judge_model,
            model_source=judge_model_source,
            system_template=judge_system_template,
        )
    elif type == "score":
        judge_config = ParametersEvaluationScoreParametersJudge(
            model=judge_model,
            model_source=judge_model_source,
            system_template=judge_system_template,
        )
    elif type == "compare":
        judge_config = ParametersEvaluationCompareParametersJudge(
            model=judge_model,
            model_source=judge_model_source,
            system_template=judge_system_template,
        )

    if judge_external_api_token:
        judge_config["external_api_token"] = judge_external_api_token
    if judge_external_base_url:
        judge_config["external_base_url"] = judge_external_base_url

    return judge_config



import json as json_lib

import click

from together import Together
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.option(
    "--region",
    required=True,
    type=str,
    help="Region to create the storage volume in",
)
@click.option(
    "--size-tib",
    required=True,
    type=int,
    help="Size of the storage volume in TiB",
)
@click.option(
    "--volume-name",
    required=True,
    type=str,
    help="Name of the storage volume",
)
@click.option(
    "--json",
    is_flag=True,
    help="Output in JSON format",
)
@click.pass_context
@handle_api_errors("Clusters Storage")
def create(ctx: click.Context, region: str, size_tib: int, volume_name: str, json: bool) -> None:
    """Create a storage volume"""
    client: Together = ctx.obj

    response = client.beta.clusters.storage.create(
        region=region,
        size_tib=size_tib,
        volume_name=volume_name,
    )

    if json:
        click.echo(json_lib.dumps(response.model_dump_json(), indent=2))
    else:
        click.echo(f"Storage volume created successfully")
        click.echo(response.volume_id)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from together import Together, AsyncTogether
from tests.utils import assert_matches_type
from together.types.chat import ChatCompletion

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCompletions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_overload_1(self, client: Together) -> None:
        completion = client.chat.completions.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
        )
        assert_matches_type(ChatCompletion, completion, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: Together) -> None:
        completion = client.chat.completions.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                    "name": "name",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            chat_template_kwargs={},
            compliance="hipaa",
            context_length_exceeded_behavior="truncate",
            echo=True,
            frequency_penalty=0,
            function_call="none",
            logit_bias={
                "105": 21.4,
                "1024": -10.5,
            },
            logprobs=0,
            max_tokens=0,
            min_p=0,
            n=1,
            presence_penalty=0,
            reasoning_effort="medium",
            repetition_penalty=0,
            response_format={"type": "text"},
            safety_model="safety_model_name",
            seed=42,
            stop=["string"],
            stream=False,
            temperature=0,
            tool_choice="tool_name",
            tools=[
                {
                    "function": {
                        "description": "A description of the function.",
                        "name": "function_name",
                        "parameters": {"foo": "bar"},
                    },
                    "type": "tool_type",
                }
            ],
            top_k=0,
            top_p=0,
        )
        assert_matches_type(ChatCompletion, completion, path=["response"])

    @parametrize
    def test_raw_response_create_overload_1(self, client: Together) -> None:
        response = client.chat.completions.with_raw_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        completion = response.parse()
        assert_matches_type(ChatCompletion, completion, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_1(self, client: Together) -> None:
        with client.chat.completions.with_streaming_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            completion = response.parse()
            assert_matches_type(ChatCompletion, completion, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_2(self, client: Together) -> None:
        completion_stream = client.chat.completions.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            stream=True,
        )
        completion_stream.response.close()

    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: Together) -> None:
        completion_stream = client.chat.completions.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                    "name": "name",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            stream=True,
            chat_template_kwargs={},
            compliance="hipaa",
            context_length_exceeded_behavior="truncate",
            echo=True,
            frequency_penalty=0,
            function_call="none",
            logit_bias={
                "105": 21.4,
                "1024": -10.5,
            },
            logprobs=0,
            max_tokens=0,
            min_p=0,
            n=1,
            presence_penalty=0,
            reasoning_effort="medium",
            repetition_penalty=0,
            response_format={"type": "text"},
            safety_model="safety_model_name",
            seed=42,
            stop=["string"],
            temperature=0,
            tool_choice="tool_name",
            tools=[
                {
                    "function": {
                        "description": "A description of the function.",
                        "name": "function_name",
                        "parameters": {"foo": "bar"},
                    },
                    "type": "tool_type",
                }
            ],
            top_k=0,
            top_p=0,
        )
        completion_stream.response.close()

    @parametrize
    def test_raw_response_create_overload_2(self, client: Together) -> None:
        response = client.chat.completions.with_raw_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_create_overload_2(self, client: Together) -> None:
        with client.chat.completions.with_streaming_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True


class TestAsyncCompletions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncTogether) -> None:
        completion = await async_client.chat.completions.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
        )
        assert_matches_type(ChatCompletion, completion, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncTogether) -> None:
        completion = await async_client.chat.completions.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                    "name": "name",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            chat_template_kwargs={},
            compliance="hipaa",
            context_length_exceeded_behavior="truncate",
            echo=True,
            frequency_penalty=0,
            function_call="none",
            logit_bias={
                "105": 21.4,
                "1024": -10.5,
            },
            logprobs=0,
            max_tokens=0,
            min_p=0,
            n=1,
            presence_penalty=0,
            reasoning_effort="medium",
            repetition_penalty=0,
            response_format={"type": "text"},
            safety_model="safety_model_name",
            seed=42,
            stop=["string"],
            stream=False,
            temperature=0,
            tool_choice="tool_name",
            tools=[
                {
                    "function": {
                        "description": "A description of the function.",
                        "name": "function_name",
                        "parameters": {"foo": "bar"},
                    },
                    "type": "tool_type",
                }
            ],
            top_k=0,
            top_p=0,
        )
        assert_matches_type(ChatCompletion, completion, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncTogether) -> None:
        response = await async_client.chat.completions.with_raw_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        completion = await response.parse()
        assert_matches_type(ChatCompletion, completion, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncTogether) -> None:
        async with async_client.chat.completions.with_streaming_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            completion = await response.parse()
            assert_matches_type(ChatCompletion, completion, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncTogether) -> None:
        completion_stream = await async_client.chat.completions.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            stream=True,
        )
        await completion_stream.response.aclose()

    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncTogether) -> None:
        completion_stream = await async_client.chat.completions.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                    "name": "name",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            stream=True,
            chat_template_kwargs={},
            compliance="hipaa",
            context_length_exceeded_behavior="truncate",
            echo=True,
            frequency_penalty=0,
            function_call="none",
            logit_bias={
                "105": 21.4,
                "1024": -10.5,
            },
            logprobs=0,
            max_tokens=0,
            min_p=0,
            n=1,
            presence_penalty=0,
            reasoning_effort="medium",
            repetition_penalty=0,
            response_format={"type": "text"},
            safety_model="safety_model_name",
            seed=42,
            stop=["string"],
            temperature=0,
            tool_choice="tool_name",
            tools=[
                {
                    "function": {
                        "description": "A description of the function.",
                        "name": "function_name",
                        "parameters": {"foo": "bar"},
                    },
                    "type": "tool_type",
                }
            ],
            top_k=0,
            top_p=0,
        )
        await completion_stream.response.aclose()

    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncTogether) -> None:
        response = await async_client.chat.completions.with_raw_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncTogether) -> None:
        async with async_client.chat.completions.with_streaming_response.create(
            messages=[
                {
                    "content": "content",
                    "role": "system",
                }
            ],
            model="meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True



from __future__ import annotations

import json as json_lib
import getpass
from typing import List, Literal

import click
from rich import print

from together import Together
from together.lib.cli.api._utils import handle_api_errors
from together.types.beta.cluster_create_params import SharedVolume, ClusterCreateParams


@click.command()
@click.option(
    "--name",
    type=str,
    help="Name of the cluster",
)
@click.option(
    "--num-gpus",
    type=int,
    help="Number of GPUs to allocate in the cluster",
)
@click.option(
    "--region",
    type=str,
    help="Region to create the cluster in",
)
@click.option(
    "--billing-type",
    type=str,
    help="Billing type to use for the cluster",
)
@click.option(
    "--driver-version",
    type=str,
    help="Driver version to use for the cluster",
)
@click.option(
    "--duration-days",
    type=int,
    help="Duration in days to keep the cluster running for reserved clusters",
)
@click.option(
    "--gpu-type",
    type=str,
    help="GPU type to use for the cluster. Find available gpu types for each region with the `list-regions` command.",
)
@click.option("--cluster-type", type=click.Choice(["KUBERNETES", "SLURM"]), help="Cluster type")
@click.option(
    "--volume",
    type=str,
    help="Storage volume ID to use for the cluster",
)
@click.option(
    "--json",
    is_flag=True,
    help="Output in JSON format",
)
@click.pass_context
@handle_api_errors("Clusters")
def create(
    ctx: click.Context,
    name: str | None = None,
    num_gpus: int | None = None,
    region: str | None = None,
    billing_type: Literal["RESERVED", "ON_DEMAND"] | None = None,
    driver_version: str | None = None,
    duration_days: int | None = None,
    gpu_type: str | None = None,
    cluster_type: Literal["KUBERNETES", "SLURM"] | None = None,
    volume: str | None = None,
    json: bool = False,
) -> None:
    """Create a cluster"""
    client: Together = ctx.obj

    params = ClusterCreateParams(
        cluster_name=name,  # type: ignore
        num_gpus=num_gpus,  # type: ignore
        region=region,  # type: ignore
        billing_type=billing_type,  # type: ignore
        driver_version=driver_version,  # type: ignore
        duration_days=duration_days,  # type: ignore
        gpu_type=gpu_type,  # type: ignore
        cluster_type=cluster_type,  # type: ignore
    )

    # Lazily add this so its not put in the object as None - just looks bad aesthetically
    if volume:
        params["volume_id"] = volume

    # JSON Mode skips hand holding through the argument setup
    if not json:
        if not name:
            params["cluster_name"] = click.prompt("Clusters: Cluster name:", default=getpass.getuser(), type=str)

        # TODO
        # GPU should be queried first
        # Validate region has the gpu selected.

        if not gpu_type:
            # TODO: Pull GPUS from region list and the region selected.
            # TODO: Add instance_types to region list api
            params["gpu_type"] = click.prompt(
                "Clusters: Cluster GPU type:",
                type=click.Choice(["H100_SXM", "H200_SXM", "RTX_6000_PCI", "L40_PCIE", "B200_SXM", "H100_SXM_INF"]),
            )

        if not region:
            regions = client.beta.clusters.list_regions()
            params["region"] = click.prompt(
                "Clusters: Cluster region:",
                default=regions.regions[0].name,
                type=click.Choice([region.name for region in regions.regions]),
            )

        if num_gpus is None:
            params["num_gpus"] = click.prompt("Clusters: Cluster GPUs count", type=click.IntRange(min=8, max=64))

        if not billing_type:
            params["billing_type"] = click.prompt(
                "Clusters: Cluster billing type:", default="ON_DEMAND", type=click.Choice(["RESERVED", "ON_DEMAND"])
            )

        if not driver_version:
            regions = client.beta.clusters.list_regions()

            # Get the driver versions for the selected region
            driver_versions: List[str] = []
            for region_obj in regions.regions:
                if region_obj.name == params["region"]:
                    driver_versions.extend(region_obj.driver_versions)

            params["driver_version"] = click.prompt(
                "Clusters: Cluster driver version:", default="CUDA_12_5_555", type=click.Choice(driver_versions)
            )

        if not duration_days and params["billing_type"] == "RESERVED":
            params["duration_days"] = click.prompt("Clusters: Cluster reserved duration (1-90 days):", default=3)

        if not cluster_type:
            params["cluster_type"] = click.prompt(
                "Clusters: Cluster type:", default="KUBERNETES", type=click.Choice(["KUBERNETES", "SLURM"])
            )

        # In our QA environment, we don't accept storage volume creation, so we skip the prompt
        if not volume and "qa" not in client.base_url.host:
            if click.confirm("Clusters: Create a new storage volume?"):
                default_volume_name = f"{params['cluster_name']}-storage"
                params["shared_volume"] = SharedVolume(
                    region=f"{params['region']}",
                    size_tib=1,
                    volume_name=default_volume_name,
                )
                params["shared_volume"]["volume_name"] = click.prompt(
                    "Clusters: Storage volume name:", default=default_volume_name, type=str
                )
                params["shared_volume"]["size_tib"] = click.prompt(
                    "Clusters: Storage volume size (TiB):", default=1, type=click.IntRange(min=1, max=1024)
                )
            else:
                # TODO: We need bound status and region on the volume list from the API.
                # Only show volumes in the region selected and that are not attached to a cluster.
                volumes = client.beta.clusters.storage.list()
                params["volume_id"] = click.prompt(
                    "Clusters: Which storage volume to use?",
                    default=volumes.volumes[0].volume_id,
                    type=click.Choice([volume.volume_id for volume in volumes.volumes]),
                )

        click.echo("Clusters: Creating cluster with the following parameters:")
        print(ClusterCreateParams(**params))  # type: ignore

    response = client.beta.clusters.create(**params)

    if json:
        click.echo(json_lib.dumps(response.model_dump(exclude_none=True), indent=4))
    else:
        click.echo(f"Clusters: Cluster created successfully")
        click.echo(f"Clusters: {response.cluster_id}")



import click

from .list import list
from .upload import upload


@click.group()
@click.pass_context
def models(ctx: click.Context) -> None:
    """Models API commands"""
    pass


models.add_command(list)
models.add_command(upload)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .file_type import FileType
from .file_purpose import FilePurpose

__all__ = ["FileResponse"]


class FileResponse(BaseModel):
    id: str

    bytes: int

    created_at: int

    filename: str

    file_type: FileType = FieldInfo(alias="FileType")
    """The type of the file"""

    line_count: int = FieldInfo(alias="LineCount")

    object: str

    processed: bool = FieldInfo(alias="Processed")

    purpose: FilePurpose
    """The purpose of the file"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from together import Together, AsyncTogether
from tests.utils import assert_matches_type
from together.types.beta import (
    Cluster,
    ClusterListResponse,
    ClusterDeleteResponse,
    ClusterListRegionsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestClusters:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Together) -> None:
        cluster = client.beta.clusters.create(
            billing_type="RESERVED",
            cluster_name="cluster_name",
            driver_version="CUDA_12_5_555",
            gpu_type="H100_SXM",
            num_gpus=0,
            region="us-central-8",
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Together) -> None:
        cluster = client.beta.clusters.create(
            billing_type="RESERVED",
            cluster_name="cluster_name",
            driver_version="CUDA_12_5_555",
            gpu_type="H100_SXM",
            num_gpus=0,
            region="us-central-8",
            cluster_type="KUBERNETES",
            duration_days=0,
            shared_volume={
                "region": "region",
                "size_tib": 0,
                "volume_name": "volume_name",
            },
            volume_id="volume_id",
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Together) -> None:
        response = client.beta.clusters.with_raw_response.create(
            billing_type="RESERVED",
            cluster_name="cluster_name",
            driver_version="CUDA_12_5_555",
            gpu_type="H100_SXM",
            num_gpus=0,
            region="us-central-8",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = response.parse()
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Together) -> None:
        with client.beta.clusters.with_streaming_response.create(
            billing_type="RESERVED",
            cluster_name="cluster_name",
            driver_version="CUDA_12_5_555",
            gpu_type="H100_SXM",
            num_gpus=0,
            region="us-central-8",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = response.parse()
            assert_matches_type(Cluster, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Together) -> None:
        cluster = client.beta.clusters.retrieve(
            "cluster_id",
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Together) -> None:
        response = client.beta.clusters.with_raw_response.retrieve(
            "cluster_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = response.parse()
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Together) -> None:
        with client.beta.clusters.with_streaming_response.retrieve(
            "cluster_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = response.parse()
            assert_matches_type(Cluster, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Together) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `cluster_id` but received ''"):
            client.beta.clusters.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Together) -> None:
        cluster = client.beta.clusters.update(
            cluster_id="cluster_id",
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Together) -> None:
        cluster = client.beta.clusters.update(
            cluster_id="cluster_id",
            cluster_type="KUBERNETES",
            num_gpus=0,
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Together) -> None:
        response = client.beta.clusters.with_raw_response.update(
            cluster_id="cluster_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = response.parse()
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Together) -> None:
        with client.beta.clusters.with_streaming_response.update(
            cluster_id="cluster_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = response.parse()
            assert_matches_type(Cluster, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Together) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `cluster_id` but received ''"):
            client.beta.clusters.with_raw_response.update(
                cluster_id="",
            )

    @parametrize
    def test_method_list(self, client: Together) -> None:
        cluster = client.beta.clusters.list()
        assert_matches_type(ClusterListResponse, cluster, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Together) -> None:
        response = client.beta.clusters.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = response.parse()
        assert_matches_type(ClusterListResponse, cluster, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Together) -> None:
        with client.beta.clusters.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = response.parse()
            assert_matches_type(ClusterListResponse, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Together) -> None:
        cluster = client.beta.clusters.delete(
            "cluster_id",
        )
        assert_matches_type(ClusterDeleteResponse, cluster, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Together) -> None:
        response = client.beta.clusters.with_raw_response.delete(
            "cluster_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = response.parse()
        assert_matches_type(ClusterDeleteResponse, cluster, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Together) -> None:
        with client.beta.clusters.with_streaming_response.delete(
            "cluster_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = response.parse()
            assert_matches_type(ClusterDeleteResponse, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Together) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `cluster_id` but received ''"):
            client.beta.clusters.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_list_regions(self, client: Together) -> None:
        cluster = client.beta.clusters.list_regions()
        assert_matches_type(ClusterListRegionsResponse, cluster, path=["response"])

    @parametrize
    def test_raw_response_list_regions(self, client: Together) -> None:
        response = client.beta.clusters.with_raw_response.list_regions()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = response.parse()
        assert_matches_type(ClusterListRegionsResponse, cluster, path=["response"])

    @parametrize
    def test_streaming_response_list_regions(self, client: Together) -> None:
        with client.beta.clusters.with_streaming_response.list_regions() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = response.parse()
            assert_matches_type(ClusterListRegionsResponse, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncClusters:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncTogether) -> None:
        cluster = await async_client.beta.clusters.create(
            billing_type="RESERVED",
            cluster_name="cluster_name",
            driver_version="CUDA_12_5_555",
            gpu_type="H100_SXM",
            num_gpus=0,
            region="us-central-8",
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTogether) -> None:
        cluster = await async_client.beta.clusters.create(
            billing_type="RESERVED",
            cluster_name="cluster_name",
            driver_version="CUDA_12_5_555",
            gpu_type="H100_SXM",
            num_gpus=0,
            region="us-central-8",
            cluster_type="KUBERNETES",
            duration_days=0,
            shared_volume={
                "region": "region",
                "size_tib": 0,
                "volume_name": "volume_name",
            },
            volume_id="volume_id",
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTogether) -> None:
        response = await async_client.beta.clusters.with_raw_response.create(
            billing_type="RESERVED",
            cluster_name="cluster_name",
            driver_version="CUDA_12_5_555",
            gpu_type="H100_SXM",
            num_gpus=0,
            region="us-central-8",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = await response.parse()
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTogether) -> None:
        async with async_client.beta.clusters.with_streaming_response.create(
            billing_type="RESERVED",
            cluster_name="cluster_name",
            driver_version="CUDA_12_5_555",
            gpu_type="H100_SXM",
            num_gpus=0,
            region="us-central-8",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = await response.parse()
            assert_matches_type(Cluster, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTogether) -> None:
        cluster = await async_client.beta.clusters.retrieve(
            "cluster_id",
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTogether) -> None:
        response = await async_client.beta.clusters.with_raw_response.retrieve(
            "cluster_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = await response.parse()
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTogether) -> None:
        async with async_client.beta.clusters.with_streaming_response.retrieve(
            "cluster_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = await response.parse()
            assert_matches_type(Cluster, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTogether) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `cluster_id` but received ''"):
            await async_client.beta.clusters.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncTogether) -> None:
        cluster = await async_client.beta.clusters.update(
            cluster_id="cluster_id",
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTogether) -> None:
        cluster = await async_client.beta.clusters.update(
            cluster_id="cluster_id",
            cluster_type="KUBERNETES",
            num_gpus=0,
        )
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTogether) -> None:
        response = await async_client.beta.clusters.with_raw_response.update(
            cluster_id="cluster_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = await response.parse()
        assert_matches_type(Cluster, cluster, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTogether) -> None:
        async with async_client.beta.clusters.with_streaming_response.update(
            cluster_id="cluster_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = await response.parse()
            assert_matches_type(Cluster, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncTogether) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `cluster_id` but received ''"):
            await async_client.beta.clusters.with_raw_response.update(
                cluster_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncTogether) -> None:
        cluster = await async_client.beta.clusters.list()
        assert_matches_type(ClusterListResponse, cluster, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTogether) -> None:
        response = await async_client.beta.clusters.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = await response.parse()
        assert_matches_type(ClusterListResponse, cluster, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTogether) -> None:
        async with async_client.beta.clusters.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = await response.parse()
            assert_matches_type(ClusterListResponse, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncTogether) -> None:
        cluster = await async_client.beta.clusters.delete(
            "cluster_id",
        )
        assert_matches_type(ClusterDeleteResponse, cluster, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTogether) -> None:
        response = await async_client.beta.clusters.with_raw_response.delete(
            "cluster_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = await response.parse()
        assert_matches_type(ClusterDeleteResponse, cluster, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTogether) -> None:
        async with async_client.beta.clusters.with_streaming_response.delete(
            "cluster_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = await response.parse()
            assert_matches_type(ClusterDeleteResponse, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTogether) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `cluster_id` but received ''"):
            await async_client.beta.clusters.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_list_regions(self, async_client: AsyncTogether) -> None:
        cluster = await async_client.beta.clusters.list_regions()
        assert_matches_type(ClusterListRegionsResponse, cluster, path=["response"])

    @parametrize
    async def test_raw_response_list_regions(self, async_client: AsyncTogether) -> None:
        response = await async_client.beta.clusters.with_raw_response.list_regions()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cluster = await response.parse()
        assert_matches_type(ClusterListRegionsResponse, cluster, path=["response"])

    @parametrize
    async def test_streaming_response_list_regions(self, async_client: AsyncTogether) -> None:
        async with async_client.beta.clusters.with_streaming_response.list_regions() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cluster = await response.parse()
            assert_matches_type(ClusterListRegionsResponse, cluster, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ClusterCreateParams", "SharedVolume"]


class ClusterCreateParams(TypedDict, total=False):
    billing_type: Required[Literal["RESERVED", "ON_DEMAND"]]

    cluster_name: Required[str]
    """Name of the GPU cluster."""

    driver_version: Required[Literal["CUDA_12_5_555", "CUDA_12_6_560", "CUDA_12_6_565", "CUDA_12_8_570"]]
    """NVIDIA driver version to use in the cluster."""

    gpu_type: Required[Literal["H100_SXM", "H200_SXM", "RTX_6000_PCI", "L40_PCIE", "B200_SXM", "H100_SXM_INF"]]
    """Type of GPU to use in the cluster"""

    num_gpus: Required[int]
    """Number of GPUs to allocate in the cluster.

    This must be multiple of 8. For example, 8, 16 or 24
    """

    region: Required[Literal["us-central-8", "us-central-4"]]
    """Region to create the GPU cluster in.

    Valid values are us-central-8 and us-central-4.
    """

    cluster_type: Literal["KUBERNETES", "SLURM"]

    duration_days: int
    """Duration in days to keep the cluster running."""

    shared_volume: SharedVolume

    volume_id: str


class SharedVolume(TypedDict, total=False):
    region: Required[str]
    """Region name. Usable regions can be found from `client.clusters.list_regions()`"""

    size_tib: Required[int]
    """Volume size in whole tebibytes (TiB)."""

    volume_name: Required[str]



from __future__ import annotations

import os
import stat
import errno
import platform
from typing import Any
from pathlib import Path

import click

from together import Together, TogetherError
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.argument("cluster-id", required=True)
@click.option(
    "--file",
    default=os.path.join(os.path.expanduser("~"), ".kube", "config"),
    show_default=True,
    type=str,
    help="Path to write the kubeconfig to. If you pass `-` it will print the config to stdout instead of writing to a file.",
)
@click.option(
    "--context-name",
    type=str,
    help="Name of the context to add to the kubeconfig. By default it will be the cluster name.",
)
@click.option(
    "--overwrite-existing",
    is_flag=True,
    help="If there is a conflict with the existing kubeconfig, overwrite the existing kubeconfig instead of raising an error.",
)
@click.option(
    "--set-default-context",
    is_flag=True,
    help="Change the current context for kubectl to the new context.",
)
@click.pass_context
@handle_api_errors("Clusters")
def get_credentials(
    ctx: click.Context,
    cluster_id: str,
    file: str,
    context_name: str | None = None,
    overwrite_existing: bool = False,
    set_default_context: bool = False,
) -> None:
    """Get cluster credentials"""
    client: Together = ctx.obj

    cluster = client.beta.clusters.retrieve(cluster_id)

    import base64

    kube_config = base64.b64decode(cluster.kube_config).decode("utf-8")

    if len(kube_config) == 0:
        click.echo("No kubeconfig found for cluster at this time.")
        return

    if file == "-":
        click.echo(kube_config)
        return

    kube_config_path = Path(os.path.expanduser(file if file else "~/.kube/config"))

    # ensure that at least an empty ~/.kube/config exists
    directory = os.path.dirname(kube_config_path)
    if directory and not os.path.exists(directory):
        try:
            os.makedirs(directory)
        except OSError as ex:
            if ex.errno != errno.EEXIST:
                raise
    if not os.path.exists(kube_config_path):
        with os.fdopen(os.open(kube_config_path, os.O_CREAT | os.O_WRONLY, 0o600), "wt"):
            pass

    # Write the decoded kubeconfig to the user's default kubeconfig path
    # Ensure the .kube directory exists before writing the config file
    try:
        from yaml import dump, safe_load
    except ImportError:
        click.secho("Together cli dependencies are missing. Please run one of the following commands:\n", fg="red")
        click.secho("uv:  uv add together --optional cli", fg="yellow")
        click.secho("pip: pip install together[cli]", fg="yellow")
        return

    # Load both configs into dictionaries to merge them
    kube_config_dict: dict[str, Any] | None = safe_load(kube_config_path.read_text())
    incoming_config_dict: dict[str, Any] = safe_load(kube_config)

    # If the user did not pass a custom context name, we will use the cluster name
    if context_name is None:
        context_name = cluster.cluster_name

    # Update the context name on the incoming data.
    incoming_config_dict["contexts"][0]["name"] = context_name
    incoming_config_dict["contexts"][0]["context"]["cluster"] = context_name
    incoming_config_dict["clusters"][0]["name"] = context_name

    # If there is not a current kube config on disk, we can safely just take the incoming config
    if kube_config_dict is None:
        kube_config_dict = incoming_config_dict
    else:
        _handle_merge(kube_config_dict, incoming_config_dict, "clusters", overwrite_existing)
        _handle_merge(kube_config_dict, incoming_config_dict, "users", overwrite_existing)
        _handle_merge(kube_config_dict, incoming_config_dict, "contexts", overwrite_existing)

    # Set the current context to the new context if the user requested it.
    if set_default_context:
        kube_config_dict["current-context"] = context_name

    # check that ~/.kube/config is only read- and writable by its owner
    if platform.system() != "Windows" and not os.path.islink(kube_config_path):
        existing_file_perms = "{:o}".format(stat.S_IMODE(os.lstat(kube_config_path).st_mode))
        if not existing_file_perms.endswith("600"):
            click.echo(
                f'{kube_config_path} has permissions "{existing_file_perms}".\nIt should be readable and writable only by its owner.',
            )
            return

    with open(kube_config_path, "w+") as stream:
        stream.write(dump(kube_config_dict))

    click.secho(f"Kubeconfig written to {kube_config_path}", fg="green")


def _handle_merge(existing: dict[str, Any], addition: dict[str, Any], key: str, overwrite_existing: bool) -> None:
    """Merge the incoming kube config into the existing config for the given key."""
    if not addition.get(key, False):
        return
    if not existing.get(key):
        existing[key] = addition[key]
        return

    for i in addition[key]:
        for j in existing[key]:
            if not i.get("name", False) or not j.get("name", False):
                continue
            if i["name"] == j["name"]:
                if overwrite_existing or i == j:
                    existing[key].remove(j)
                    break
                else:
                    raise TogetherError(
                        f"A different object named {i['name']} already exists in {key} in your kubeconfig file."
                    )
        existing[key].append(i)



import json as json_lib

import click

from together import Together


@click.command()
@click.option(
    "--json",
    is_flag=True,
    help="Output in JSON format",
)
@click.pass_context
def list(ctx: click.Context, json: bool) -> None:
    """List clusters"""
    client: Together = ctx.obj

    response = client.beta.clusters.list()

    if json:
        click.echo(json_lib.dumps(response.model_dump(exclude_none=True), indent=4))
    else:
        ctx.obj.print_clusters(response.clusters)



# Manually added to minimize breaking changes from V1
from . import (
    FileList as FileList,
    FileType as FileType,
    FilePurpose as FilePurpose,
    FileResponse as FileResponse,
    FileDeleteResponse as FileDeleteResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from together import Together, AsyncTogether
from together._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSpeech:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_create_overload_1(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        speech = client.audio.speech.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            voice="voice",
        )
        assert speech.is_closed
        assert speech.json() == {"foo": "bar"}
        assert cast(Any, speech.is_closed) is True
        assert isinstance(speech, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_create_with_all_params_overload_1(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        speech = client.audio.speech.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            voice="voice",
            language="en",
            response_encoding="pcm_f32le",
            response_format="mp3",
            sample_rate=0,
            stream=False,
        )
        assert speech.is_closed
        assert speech.json() == {"foo": "bar"}
        assert cast(Any, speech.is_closed) is True
        assert isinstance(speech, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_create_overload_1(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        speech = client.audio.speech.with_raw_response.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            voice="voice",
        )

        assert speech.is_closed is True
        assert speech.http_request.headers.get("X-Stainless-Lang") == "python"
        assert speech.json() == {"foo": "bar"}
        assert isinstance(speech, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_create_overload_1(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        with client.audio.speech.with_streaming_response.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            voice="voice",
        ) as speech:
            assert not speech.is_closed
            assert speech.http_request.headers.get("X-Stainless-Lang") == "python"

            assert speech.json() == {"foo": "bar"}
            assert cast(Any, speech.is_closed) is True
            assert isinstance(speech, StreamedBinaryAPIResponse)

        assert cast(Any, speech.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_create_overload_2(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        speech_stream = client.audio.speech.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            stream=True,
            voice="voice",
        )
        speech_stream.response.close()

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_create_with_all_params_overload_2(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        speech_stream = client.audio.speech.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            stream=True,
            voice="voice",
            language="en",
            response_encoding="pcm_f32le",
            response_format="mp3",
            sample_rate=0,
        )
        speech_stream.response.close()

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_create_overload_2(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        speech_stream = client.audio.speech.with_raw_response.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            stream=True,
            voice="voice",
        )

        assert speech_stream.http_request.headers.get("X-Stainless-Lang") == "python"
        assert speech_stream.json() == {"foo": "bar"}
        assert isinstance(speech_stream, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_create_overload_2(self, client: Together, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        with client.audio.speech.with_streaming_response.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            stream=True,
            voice="voice",
        ) as speech_stream:
            assert not speech_stream.is_closed
            assert speech_stream.http_request.headers.get("X-Stainless-Lang") == "python"

            assert speech_stream.json() == {"foo": "bar"}
            assert cast(Any, speech_stream.is_closed) is True
            assert isinstance(speech_stream, StreamedBinaryAPIResponse)

        assert cast(Any, speech_stream.is_closed) is True


class TestAsyncSpeech:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_create_overload_1(self, async_client: AsyncTogether, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        speech = await async_client.audio.speech.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            voice="voice",
        )
        assert speech.is_closed
        assert await speech.json() == {"foo": "bar"}
        assert cast(Any, speech.is_closed) is True
        assert isinstance(speech, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_create_with_all_params_overload_1(
        self, async_client: AsyncTogether, respx_mock: MockRouter
    ) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        speech = await async_client.audio.speech.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            voice="voice",
            language="en",
            response_encoding="pcm_f32le",
            response_format="mp3",
            sample_rate=0,
            stream=False,
        )
        assert speech.is_closed
        assert await speech.json() == {"foo": "bar"}
        assert cast(Any, speech.is_closed) is True
        assert isinstance(speech, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_create_overload_1(self, async_client: AsyncTogether, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        speech = await async_client.audio.speech.with_raw_response.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            voice="voice",
        )

        assert speech.is_closed is True
        assert speech.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await speech.json() == {"foo": "bar"}
        assert isinstance(speech, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_create_overload_1(
        self, async_client: AsyncTogether, respx_mock: MockRouter
    ) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        async with async_client.audio.speech.with_streaming_response.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            voice="voice",
        ) as speech:
            assert not speech.is_closed
            assert speech.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await speech.json() == {"foo": "bar"}
            assert cast(Any, speech.is_closed) is True
            assert isinstance(speech, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, speech.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_create_overload_2(self, async_client: AsyncTogether, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        speech_stream = await async_client.audio.speech.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            stream=True,
            voice="voice",
        )
        await speech_stream.response.aclose()

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_create_with_all_params_overload_2(
        self, async_client: AsyncTogether, respx_mock: MockRouter
    ) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        speech_stream = await async_client.audio.speech.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            stream=True,
            voice="voice",
            language="en",
            response_encoding="pcm_f32le",
            response_format="mp3",
            sample_rate=0,
        )
        await speech_stream.response.aclose()

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_create_overload_2(self, async_client: AsyncTogether, respx_mock: MockRouter) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        speech_stream = await async_client.audio.speech.with_raw_response.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            stream=True,
            voice="voice",
        )

        assert speech_stream.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await speech_stream.json() == {"foo": "bar"}
        assert isinstance(speech_stream, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_create_overload_2(
        self, async_client: AsyncTogether, respx_mock: MockRouter
    ) -> None:
        respx_mock.post("/audio/speech").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        async with async_client.audio.speech.with_streaming_response.create(
            input="input",
            model="canopylabs/orpheus-3b-0.1-ft",
            stream=True,
            voice="voice",
        ) as speech_stream:
            assert not speech_stream.is_closed
            assert speech_stream.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await speech_stream.json() == {"foo": "bar"}
            assert cast(Any, speech_stream.is_closed) is True
            assert isinstance(speech_stream, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, speech_stream.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["FileDeleteResponse"]


class FileDeleteResponse(BaseModel):
    id: Optional[str] = None

    deleted: Optional[bool] = None



from typing import Any, Dict, List

import click
from tabulate import tabulate

from together.types.beta.cluster import Cluster

from .list import list
from .create import create
from .delete import delete
from .update import update
from .storage import storage
from .retrieve import retrieve
from .list_regions import list_regions
from .get_credentials import get_credentials


@click.group()
@click.pass_context
def clusters(ctx: click.Context) -> None:
    """Clusters API commands"""

    def _print_clusters(clusters: List[Cluster]) -> None:
        data: List[Dict[str, Any]] = []
        for cluster in clusters:
            data.append(
                {
                    "ID": cluster.cluster_id,
                    "Name": cluster.cluster_name,
                    "Status": cluster.status,
                    "Region": cluster.region,
                }
            )
        click.echo(tabulate(data, headers="keys", tablefmt="grid"))

    ctx.obj.print_clusters = _print_clusters
    pass


clusters.add_command(list)
clusters.add_command(storage)
clusters.add_command(create)
clusters.add_command(retrieve)
clusters.add_command(update)
clusters.add_command(delete)
clusters.add_command(list_regions)
clusters.add_command(get_credentials)



from __future__ import annotations

import os
import re
import sys
import logging
from typing import Any, Set, Dict

logger: logging.Logger = logging.getLogger("together")

TOGETHER_LOG = os.environ.get("TOGETHER_LOG")

WARNING_MESSAGES_ONCE: Set[str] = set()


def _console_log_level() -> str | None:
    if TOGETHER_LOG in ["debug", "info"]:
        return TOGETHER_LOG
    else:
        return None


def logfmt(props: Dict[str, Any]) -> str:
    def fmt(key: str, val: Any) -> str:
        # Handle case where val is a bytes or bytesarray
        if hasattr(val, "decode"):
            val = val.decode("utf-8")
        # Check if val is already a string to avoid re-encoding into ascii.
        if not isinstance(val, str):
            val = str(val)
        if re.search(r"\s", val):
            val = repr(val)
        # key should already be a string
        if re.search(r"\s", key):
            key = repr(key)
        return "{key}={val}".format(key=key, val=val)

    return " ".join([fmt(key, val) for key, val in sorted(props.items())])


def log_debug(message: str | Any, **params: Any) -> None:
    msg = logfmt(dict(message=message, **params))
    if _console_log_level() == "debug":
        print(msg, file=sys.stderr)  # noqa
    logger.debug(msg)


def log_info(message: str | Any, **params: Any) -> None:
    msg = logfmt(dict(message=message, **params))
    if _console_log_level() in ["debug", "info"]:
        print(msg, file=sys.stderr)  # noqa
    logger.info(msg)


def log_warn(message: str | Any, **params: Any) -> None:
    msg = logfmt(dict(message=message, **params))
    print(msg, file=sys.stderr)  # noqa
    logger.warning(msg)


def log_warn_once(message: str | Any, **params: Any) -> None:
    msg = logfmt(dict(message=message, **params))
    if msg not in WARNING_MESSAGES_ONCE:
        print(msg, file=sys.stderr)  # noqa
        logger.warning(msg)
        WARNING_MESSAGES_ONCE.add(msg)



from __future__ import annotations

import sys

import click
from rich import print

from together import APIError, Together, omit
from together.lib.cli.api._utils import handle_api_errors

from .hardware import hardware


@click.command()
@click.option(
    "--model",
    required=True,
    help="The model to deploy (e.g. meta-llama/Llama-4-Scout-17B-16E-Instruct)",
)
@click.option(
    "--min-replicas",
    type=int,
    default=1,
    help="Minimum number of replicas to deploy",
)
@click.option(
    "--max-replicas",
    type=int,
    default=1,
    help="Maximum number of replicas to deploy",
)
@click.option(
    "--gpu",
    type=click.Choice(["b200", "h200", "h100", "a100", "l40", "l40s", "rtx-6000"]),
    required=True,
    help="GPU type to use for inference",
)
@click.option(
    "--gpu-count",
    type=int,
    default=1,
    help="Number of GPUs to use per replica",
)
@click.option(
    "--display-name",
    help="A human-readable name for the endpoint",
)
@click.option(
    "--no-prompt-cache",
    is_flag=True,
    help="Deprecated and no longer has any effect.",
)
@click.option(
    "--no-speculative-decoding",
    is_flag=True,
    help="Disable speculative decoding for this endpoint",
)
@click.option(
    "--no-auto-start",
    is_flag=True,
    help="Create the endpoint in STOPPED state instead of auto-starting it",
)
@click.option(
    "--inactive-timeout",
    type=int,
    help="Number of minutes of inactivity after which the endpoint will be automatically stopped. Set to 0 to disable.",
)
@click.option(
    "--availability-zone",
    help="Start endpoint in specified availability zone (e.g., us-central-4b)",
)
@click.option(
    "--wait/--no-wait",
    default=True,
    help="Wait for the endpoint to be ready after creation",
)
@click.pass_context
@handle_api_errors("Endpoints")
def create(
    ctx: click.Context,
    model: str,
    min_replicas: int,
    max_replicas: int,
    gpu: str,
    gpu_count: int,
    display_name: str | None,
    no_prompt_cache: bool | None,
    no_speculative_decoding: bool | None,
    no_auto_start: bool,
    inactive_timeout: int | None,
    availability_zone: str | None,
    wait: bool,
) -> None:
    """Create a new dedicated inference endpoint."""
    client: Together = ctx.obj
    # Map GPU types to their full hardware ID names
    gpu_map = {
        "b200": "nvidia_b200_180gb_sxm",
        "h200": "nvidia_h200_140gb_sxm",
        "h100": "nvidia_h100_80gb_sxm",
        "a100": "nvidia_a100_80gb_pcie" if gpu_count == 1 else "nvidia_a100_80gb_sxm",
        "l40": "nvidia_l40",
        "l40s": "nvidia_l40s",
        "rtx-6000": "nvidia_rtx_6000_ada",
    }

    if no_prompt_cache is not None:
        click.echo("Warning: --no-prompt-cache is deprecated and no longer has any effect.", err=True)

    hardware_id = f"{gpu_count}x_{gpu_map[gpu]}"

    try:
        response = client.endpoints.create(
            model=model,
            hardware=hardware_id,
            autoscaling={
                "min_replicas": min_replicas,
                "max_replicas": max_replicas,
            },
            display_name=display_name or omit,
            disable_speculative_decoding=no_speculative_decoding or omit,
            state="STOPPED" if no_auto_start else "STARTED",
            inactive_timeout=inactive_timeout,
            extra_query={"availability_zone": availability_zone or omit},
        )
    except APIError as e:
        if "check the hardware api" in str(e.args[0]).lower() or "invalid hardware provided" in str(e.args[0]).lower():
            print("Invalid hardware provided")
            ctx.invoke(hardware, available=True, model=model, json=False)
            sys.exit(1)
        raise e

    # Print detailed information to stderr
    click.echo("Created dedicated endpoint with:", err=True)
    click.echo(f"  Model: {model}", err=True)
    click.echo(f"  Min replicas: {min_replicas}", err=True)
    click.echo(f"  Max replicas: {max_replicas}", err=True)
    click.echo(f"  Hardware: {hardware_id}", err=True)
    if display_name:
        click.echo(f"  Display name: {display_name}", err=True)
    if no_speculative_decoding:
        click.echo("  Speculative decoding: disabled", err=True)
    if no_auto_start:
        click.echo("  Auto-start: disabled", err=True)
    if inactive_timeout is not None:
        click.echo(f"  Inactive timeout: {inactive_timeout} minutes", err=True)
    if availability_zone:
        click.echo(f"  Availability zone: {availability_zone}", err=True)

    click.echo(f"Endpoint created successfully, id: {response.id}", err=True)

    if wait:
        import time

        click.echo("Waiting for endpoint to be ready...", err=True)
        while client.endpoints.retrieve(response.id).state != "STARTED":
            time.sleep(1)
        click.echo("Endpoint ready", err=True)

    # Print only the endpoint ID to stdout
    click.echo(response.id)



from __future__ import annotations

from typing import Any
from datetime import datetime


def datetime_serializer(obj: Any) -> str:
    if isinstance(obj, datetime):
        return obj.isoformat()
    raise TypeError(f"Object of type {obj.__class__.__name__} is not JSON serializable")



import json

import click

from together import Together
from together.lib.cli.api._utils import handle_api_errors


@click.command()
@click.pass_context
@click.argument("fine_tune_id", type=str, required=True)
@click.option("--force", is_flag=True, help="Force deletion without confirmation")
@click.option("--quiet", is_flag=True, help="Do not prompt for confirmation before deleting job")
@handle_api_errors("Fine-tuning")
def delete(ctx: click.Context, fine_tune_id: str, force: bool = False, quiet: bool = False) -> None:
    """Delete fine-tuning job"""
    client: Together = ctx.obj

    if not quiet:
        confirm_response = input(
            f"Are you sure you want to delete fine-tuning job {fine_tune_id}? This action cannot be undone. [y/N] "
        )
        if confirm_response.lower() != "y":
            click.echo("Deletion cancelled")
            return

    response = client.fine_tuning.delete(fine_tune_id, force=force)

    click.echo(json.dumps(response.model_dump(exclude_none=True), indent=4))



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..types import video_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.video_job import VideoJob

__all__ = ["VideosResource", "AsyncVideosResource"]


class VideosResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VideosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return VideosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VideosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return VideosResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: str,
        fps: int | Omit = omit,
        frame_images: Iterable[video_create_params.FrameImage] | Omit = omit,
        guidance_scale: int | Omit = omit,
        height: int | Omit = omit,
        negative_prompt: str | Omit = omit,
        output_format: Literal["MP4", "WEBM"] | Omit = omit,
        output_quality: int | Omit = omit,
        prompt: str | Omit = omit,
        reference_images: SequenceNotStr[str] | Omit = omit,
        seconds: str | Omit = omit,
        seed: int | Omit = omit,
        steps: int | Omit = omit,
        width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VideoJob:
        """
        Create a video

        Args:
          model: The model to be used for the video creation request.

          fps: Frames per second. Defaults to 24.

          frame_images: Array of images to guide video generation, similar to keyframes.

          guidance_scale: Controls how closely the video generation follows your prompt. Higher values
              make the model adhere more strictly to your text description, while lower values
              allow more creative freedom. guidence_scale affects both visual content and
              temporal consistency.Recommended range is 6.0-10.0 for most video models. Values
              above 12 may cause over-guidance artifacts or unnatural motion patterns.

          negative_prompt: Similar to prompt, but specifies what to avoid instead of what to include

          output_format: Specifies the format of the output video. Defaults to MP4.

          output_quality: Compression quality. Defaults to 20.

          prompt: Text prompt that describes the video to generate.

          reference_images: Unlike frame_images which constrain specific timeline positions, reference
              images guide the general appearance that should appear consistently across the
              video.

          seconds: Clip duration in seconds.

          seed: Seed to use in initializing the video generation. Using the same seed allows
              deterministic video generation. If not provided a random seed is generated for
              each request.

          steps: The number of denoising steps the model performs during video generation. More
              steps typically result in higher quality output but require longer processing
              time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/videos" if self._client._base_url_overridden else "https://api.together.xyz/v2/videos",
            body=maybe_transform(
                {
                    "model": model,
                    "fps": fps,
                    "frame_images": frame_images,
                    "guidance_scale": guidance_scale,
                    "height": height,
                    "negative_prompt": negative_prompt,
                    "output_format": output_format,
                    "output_quality": output_quality,
                    "prompt": prompt,
                    "reference_images": reference_images,
                    "seconds": seconds,
                    "seed": seed,
                    "steps": steps,
                    "width": width,
                },
                video_create_params.VideoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VideoJob,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VideoJob:
        """
        Fetch video metadata

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/videos/{id}" if self._client._base_url_overridden else f"https://api.together.xyz/v2/videos/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VideoJob,
        )


class AsyncVideosResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVideosResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncVideosResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVideosResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncVideosResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: str,
        fps: int | Omit = omit,
        frame_images: Iterable[video_create_params.FrameImage] | Omit = omit,
        guidance_scale: int | Omit = omit,
        height: int | Omit = omit,
        negative_prompt: str | Omit = omit,
        output_format: Literal["MP4", "WEBM"] | Omit = omit,
        output_quality: int | Omit = omit,
        prompt: str | Omit = omit,
        reference_images: SequenceNotStr[str] | Omit = omit,
        seconds: str | Omit = omit,
        seed: int | Omit = omit,
        steps: int | Omit = omit,
        width: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VideoJob:
        """
        Create a video

        Args:
          model: The model to be used for the video creation request.

          fps: Frames per second. Defaults to 24.

          frame_images: Array of images to guide video generation, similar to keyframes.

          guidance_scale: Controls how closely the video generation follows your prompt. Higher values
              make the model adhere more strictly to your text description, while lower values
              allow more creative freedom. guidence_scale affects both visual content and
              temporal consistency.Recommended range is 6.0-10.0 for most video models. Values
              above 12 may cause over-guidance artifacts or unnatural motion patterns.

          negative_prompt: Similar to prompt, but specifies what to avoid instead of what to include

          output_format: Specifies the format of the output video. Defaults to MP4.

          output_quality: Compression quality. Defaults to 20.

          prompt: Text prompt that describes the video to generate.

          reference_images: Unlike frame_images which constrain specific timeline positions, reference
              images guide the general appearance that should appear consistently across the
              video.

          seconds: Clip duration in seconds.

          seed: Seed to use in initializing the video generation. Using the same seed allows
              deterministic video generation. If not provided a random seed is generated for
              each request.

          steps: The number of denoising steps the model performs during video generation. More
              steps typically result in higher quality output but require longer processing
              time.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/videos" if self._client._base_url_overridden else "https://api.together.xyz/v2/videos",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "fps": fps,
                    "frame_images": frame_images,
                    "guidance_scale": guidance_scale,
                    "height": height,
                    "negative_prompt": negative_prompt,
                    "output_format": output_format,
                    "output_quality": output_quality,
                    "prompt": prompt,
                    "reference_images": reference_images,
                    "seconds": seconds,
                    "seed": seed,
                    "steps": steps,
                    "width": width,
                },
                video_create_params.VideoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VideoJob,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VideoJob:
        """
        Fetch video metadata

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/videos/{id}" if self._client._base_url_overridden else f"https://api.together.xyz/v2/videos/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VideoJob,
        )


class VideosResourceWithRawResponse:
    def __init__(self, videos: VideosResource) -> None:
        self._videos = videos

        self.create = to_raw_response_wrapper(
            videos.create,
        )
        self.retrieve = to_raw_response_wrapper(
            videos.retrieve,
        )


class AsyncVideosResourceWithRawResponse:
    def __init__(self, videos: AsyncVideosResource) -> None:
        self._videos = videos

        self.create = async_to_raw_response_wrapper(
            videos.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            videos.retrieve,
        )


class VideosResourceWithStreamingResponse:
    def __init__(self, videos: VideosResource) -> None:
        self._videos = videos

        self.create = to_streamed_response_wrapper(
            videos.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            videos.retrieve,
        )


class AsyncVideosResourceWithStreamingResponse:
    def __init__(self, videos: AsyncVideosResource) -> None:
        self._videos = videos

        self.create = async_to_streamed_response_wrapper(
            videos.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            videos.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["SessionListResponse", "Data", "DataSession"]


class DataSession(BaseModel):
    id: str
    """Session Identifier. Used to make follow-up calls."""

    execute_count: int

    expires_at: datetime

    last_execute_at: datetime

    started_at: datetime


class Data(BaseModel):
    sessions: List[DataSession]


class SessionListResponse(BaseModel):
    data: Optional[Data] = None

    errors: Optional[List[Union[str, Dict[str, object]]]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.code_interpreter.session_list_response import SessionListResponse

__all__ = ["SessionsResource", "AsyncSessionsResource"]


class SessionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return SessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return SessionsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionListResponse:
        """Lists all your currently active sessions."""
        return self._get(
            "/tci/sessions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionListResponse,
        )


class AsyncSessionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSessionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/togethercomputer/together-py#accessing-raw-response-data-eg-headers
        """
        return AsyncSessionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSessionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/togethercomputer/together-py#with_streaming_response
        """
        return AsyncSessionsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SessionListResponse:
        """Lists all your currently active sessions."""
        return await self._get(
            "/tci/sessions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SessionListResponse,
        )


class SessionsResourceWithRawResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.list = to_raw_response_wrapper(
            sessions.list,
        )


class AsyncSessionsResourceWithRawResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.list = async_to_raw_response_wrapper(
            sessions.list,
        )


class SessionsResourceWithStreamingResponse:
    def __init__(self, sessions: SessionsResource) -> None:
        self._sessions = sessions

        self.list = to_streamed_response_wrapper(
            sessions.list,
        )


class AsyncSessionsResourceWithStreamingResponse:
    def __init__(self, sessions: AsyncSessionsResource) -> None:
        self._sessions = sessions

        self.list = async_to_streamed_response_wrapper(
            sessions.list,
        )
